<?php

namespace Drupal\graphql_fragment_include\GraphQL\Execution;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\Context\CacheContextsManager;
use Drupal\graphql\GraphQL\Execution\QueryProcessor as BaseQueryProcessor;
use Drupal\graphql\Plugin\SchemaPluginManager;
use Drupal\graphql_fragment_include\GraphQL\Fragment\GraphQLFragmentLoaderInterface;
use GraphQL\Executor\Promise\PromiseAdapter;
use GraphQL\Language\AST\DocumentNode;
use GraphQL\Server\OperationParams;
use GraphQL\Server\ServerConfig;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * QueryProcessor service overriding BaseQueryProcessor.
 *
 * This class overrides both executeCacheableOperation() and
 * executeUncacheableOperation() to be able to support persisted and
 * non-persisted querries.
 */
class QueryProcessor extends BaseQueryProcessor {

  /**
   * GraphQL fragment loader.
   *
   * @var \Drupal\graphql_fragment_include\GraphQL\Fragment\GraphQLFragmentLoaderInterface
   */
  protected $graphQLFragmentLoader;

  /**
   * Processor constructor.
   *
   * @param \Drupal\Core\Cache\Context\CacheContextsManager $contextsManager
   *   The cache contexts manager service.
   * @param \Drupal\graphql\Plugin\SchemaPluginManager $pluginManager
   *   The schema plugin manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   The cache backend for caching query results.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\graphql_fragment_include\GraphQL\Fragment\GraphQLFragmentLoaderInterface $graphQLFragmentLoader
   *   The GraphQL fragment loader.
   */
  public function __construct(
    CacheContextsManager $contextsManager,
    SchemaPluginManager $pluginManager,
    CacheBackendInterface $cacheBackend,
    RequestStack $requestStack,
    GraphQLFragmentLoaderInterface $graphQLFragmentLoader
  ) {
    parent::__construct($contextsManager, $pluginManager, $cacheBackend, $requestStack);
    $this->graphQLFragmentLoader = $graphQLFragmentLoader;
  }

  /**
   * {@inheritdoc}
   */
  protected function executeCacheableOperation(PromiseAdapter $adapter, ServerConfig $config, OperationParams $params, DocumentNode $document, $validate = TRUE) {
    // Load GraphQL fragments.
    if ($params->query) {
      $params->query = $this->graphQLFragmentLoader->loadFragments($params->query);
    }

    return parent::executeCacheableOperation($adapter, $config, $params, $document, $validate);
  }

  /**
   * {@inheritdoc}
   */
  protected function executeUncachableOperation(PromiseAdapter $adapter, ServerConfig $config, OperationParams $params, DocumentNode $document, $validate = TRUE) {
    // Load GraphQL fragments.
    if ($params->query) {
      $params->query = $this->graphQLFragmentLoader->loadFragments($params->query);
    }

    return parent::executeUncachableOperation($adapter, $config, $params, $document, $validate);
  }

}
